﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls.GridView;
using System.ComponentModel;

namespace RadTreeListViewDragAndDropSample_WPF
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
	
			InitializeComponent();
			this.DataContext = SampleTreeDataProvider.GetSampleData();
			this.treeView.DataLoaded += new EventHandler<EventArgs>(treeView_DataLoaded);
			this.treeView.AddHandler(RadDragAndDropManager.DropInfoEvent,
				new EventHandler<DragDropEventArgs>(treeList_OnDropInfo));
			this.treeView.AddHandler(RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(treeList_OnDropQuery));
		}
		void treeView_DataLoaded(object sender, EventArgs e)
		{
			this.treeView.DataLoaded -= new EventHandler<EventArgs>(treeView_DataLoaded);
			this.treeView.ExpandAllHierarchyItems();
		}
		void treeList_OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
		
		}

		void treeList_OnDropInfo(object sender, DragDropEventArgs e)
		{
			var dragCue = e.Options.DragCue as TreeListViewDragCue;

			if (dragCue != null)
			{
				dragCue.DragTooltipContent = "hello";
			}

			if (e.Options.Status == DragStatus.DropComplete)
			{
				Collection<Object> payload = e.Options.Payload as Collection<Object>;
				FileItem droppedItem = (FileItem)payload[0];
				GridViewRow destinationRow = e.Options.Destination as GridViewRow;
				FileItem targetItem = destinationRow.DataContext as FileItem;
				var relativeDropPosition = (TreeListViewDropPosition)destinationRow.GetValue(RadTreeListView.DropPositionProperty);

				MoveItem(droppedItem, targetItem, relativeDropPosition);
			}
		}

		private void MoveItem(FileItem droppedItem, FileItem targetItem, TreeListViewDropPosition relativeDropPosition)
		{
			var sourceCollection = (ObservableCollection<FileItem>)this.treeView.ItemsSource;
			var parentCollection = GetParentCollection(sourceCollection, droppedItem);

			parentCollection.Remove(droppedItem);

			if (relativeDropPosition == TreeListViewDropPosition.Inside)
			{
				targetItem.SubItems.Add(droppedItem);
			}
			else if (relativeDropPosition == TreeListViewDropPosition.Before)
			{
				var targetParentCollection = GetParentCollection(sourceCollection, targetItem);
				targetParentCollection.Insert(targetParentCollection.IndexOf(targetItem), droppedItem);
			}
			else if (relativeDropPosition == TreeListViewDropPosition.After)
			{
				var targetParentCollection = GetParentCollection(sourceCollection, targetItem);
				targetParentCollection.Insert(targetParentCollection.IndexOf(targetItem) + 1, droppedItem);
			}
		}

		private ObservableCollection<FileItem> GetParentCollection(ObservableCollection<FileItem> items, FileItem item)
		{
			if (items.Contains(item))
			{
				return items;
			}
			else
			{
				foreach (var subitem in items)
				{
					var collection = GetParentCollection(subitem.SubItems, item);
					if (collection != null)
						return collection;
				}
			}

			return null;
		}

		private void Button_Click(object sender, RoutedEventArgs e)
		{
			SampleTreeDataProvider.AddFolder2();
		}
	}

	public class FileItem :INotifyPropertyChanged
	{
		public FileItem(bool isFolder, String description)
		{
			this.subItems = new ObservableCollection<FileItem>();
			this.Description = description;
			this.IsFolder = isFolder;
			this.LastModified = DateTime.Now;
		}

		public string Description { get; set; }
		public bool IsFolder { get; set; }

		private ObservableCollection<FileItem> subItems;
		public ObservableCollection<FileItem> SubItems
		{
			get
			{
				return this.subItems;
			}
		}

		public DateTime LastModified { get; set; }

		public event PropertyChangedEventHandler PropertyChanged;
	}
}
